using System;
using System.Collections.Generic;
using System.Text;
using FarPoint.Win.Spread.Model;
using FarPoint.Win.Spread;


namespace SpreadCustomDataModel
{   // prototype class to play around with custom data models
    class ComputedDataModel : DefaultSheetDataModel
    {   
        bool mTranspose;
        SheetView mSV;
        public ComputedDataModel(bool transpose, SheetView sv)  // added SheetView to support GetModelColumnFromViewColumn() method
            : base(DataStore.ROWS, DataStore.ROWS)      // keep getting System.IndexOutOfRangeException for any small values like (10,10); not sure why
        {
            mTranspose = transpose;
            mSV = sv;
        }

         public override int RowCount
        {
            get
            {
                if (mTranspose) return DataStore.COLS;
                else            return DataStore.ROWS;
            }
        }

        public override int ColumnCount
        {
            get
            {
                if (!mTranspose)    return DataStore.COLS;
                else                return DataStore.ROWS;
            }
        }
        public override object GetValue(int row, int column)
        {
            int col = column;
            col = mSV.GetModelColumnFromViewColumn(column); // should remap columns if user moves them
            
            if (mTranspose)
                return DataStore.data[col][row];
            else
                return DataStore.data[row][col];
        }
      
        public override void SetValue(int row, int column, object value)
        {
            int col = column;
            col = mSV.GetModelColumnFromViewColumn(column);
            int v = Convert.ToInt32(value);
            if (mTranspose)
                DataStore.data[col][row] = v;
            else
                DataStore.data[row][col] = v;
        }
    }

    /// <summary>
    /// simple data storage class to test sharing data between multiple spreadsheets using custom data models
    /// </summary>
    static class DataStore
    {
        public static int[][] data;
        public const int ROWS=250,COLS=10;
        static DataStore()
        {
            data = new int[ROWS][];
            for (int i = 0; i < data.Length; i++)
            {
                int[] x = new int[COLS];
                for (int j = 0; j < x.Length; j++)
                    x[j] = i + j;
                data[i] = x;
            }
        }
    }
}
